/**
   build with:

  g++ -Wall -g -O0 -I../build/src/ES -I../interface -o createActivity createActivity.cpp -L../../stage/usr/lib64 -lglite_ce_es_client -lgridsite -lgsoap++ -lglite_security_gsoap_plugin_2713_cxx
  
*/

#include "glite/ce/es-client-api-c/ActivityDescription.h"
#include "glite/ce/es-client-api-c/CreateActivities.h"
#include "glite/ce/es-client-api-c/CreateActivitiesResponse.h"
#include "glite/ce/es-client-api-c/CreateActivityDescriptionFromXML.h"
#include "glite/ce/es-client-api-c/CreateActivityCall.h"
#include "glite/ce/es-client-api-c/CallFactory.h"
#include "glite/ce/es-client-api-c/util.h"

#include <string>
#include <vector>
#include <utility>
#include <iostream>
#include <sys/time.h>

#include <boost/lexical_cast.hpp>

using namespace std;
using namespace emi_es::client::wrapper;
using namespace emi_es::client::comm;
using namespace emi_es::client;
using namespace emi_es::client::util;

SOAP_NMAC struct Namespace namespaces[] = {};

void soap_serialize_xsd__QName(soap*, 
			       std::basic_string<char, std::char_traits<char>, 
			       std::allocator<char> > const*) 
{}

//------------------------------------------------------------------------------
int main( int argc, char* argv[] ) {
  if ( argc < 3 )
    return 0;

  string endpoint = argv[1];
  string xmlfile  = argv[2];

  //while(1) {
    vector<ActivityDescription> caVec;
    
    vector< pair<ActivityDescription*, string> > actDesc;
    try {
      CreateActivityDescriptionFromXML::create( xmlfile, actDesc );
    } catch(string& ex) {
      cerr << ex << endl;
      return 1;
    }
    vector< pair<ActivityDescription*, string> >::const_iterator it;
    it = actDesc.begin();
    for( ; it != actDesc.end(); ++it ) {
      ActivityDescription* current = it->first;
      if(!current) {
	cerr << it->second << endl;
	continue;
      }
      caVec.push_back( *current );
    }
    
    CreateActivities CA( caVec ); 
    
    AbstractCall* call = CallFactory::makeCall( endpoint, &CA );
    
    string error;
    
    uid_t userID = ::getuid( );
    
    string usercert = string("/tmp/x509up_u") + boost::lexical_cast<string>( userID );
    string userkey  = string("/tmp/x509up_u") + boost::lexical_cast<string>( userID );
   
//    cout << "usercert = " << usercert << endl;
 
    if ( !::getenv("NOAUTH") ) {
      if(!call->init( "/tmp/x509up_u501", "/tmp/x509up_u501", error ))
	{
	  cerr << error << endl;
	  return 1;
	}
    } else {
      if(!call->init("", "", error ))
	{
	  cerr << error << endl;
	  return 1;
	}
    }
    
    emi_es::client::comm::AbstractCall::SOAP_CALL_ERROR_CODES code;
    if(!call->execute( error, code ) ) {
      cerr << error << endl;
      //cerr << AbstractCall::SOAP_CALL_ERROR_CODES_STRING[code] << endl;
      return 1;
    }
    CreateActivityDescriptionFromXML::free( actDesc );
    
    vector<ActivityCreationResponse> resps;
    call->getResponse( resps );
    
    vector<ActivityCreationResponse>::const_iterator ait = resps.begin( );
    for( ; ait != resps.end( ); ++ait )
      {
	cout << "*****************************************\nActivityID     = " 
	     << ait->getActivityID( ) 
	     << endl;
	if( ait->getFault( ) )
	  {
	    string err;
	    AbstractCall::get_string_error( err, *ait->getFault( ) );
	    cerr << "FAULT: " << err << endl;
	  }
	cout << "ActivityMgrURI = " << ait->getActivityManagerURI( ) << endl;
	cout << "Status         = " 
	     << ait->getActivityStatus( )->ACTIVITYSTATUS_STRING[ait->getActivityStatus( )->getStatusNum()]<<endl;
	vector<string> tgt;
	ait->getActivityStatus( )->getStatusAttributesString( tgt );
	cout << "Status Attrs   = {" << join( tgt, ", " ) << "}" << endl;
	
	cout << "Timestamp      = " << ait->getActivityStatus( )->getTimestampString( ) << endl;
	cout << "Description    = " << ait->getActivityStatus( )->getDescription() << endl;
	cout << "ETNSC          = " << ait->getETNSCString( )<<endl;
	cout << "STAGEIN Dir    = {" << join(ait->getStageInDirectory(), ", ") << "}"<<endl;
	cout << "SESSION Dir    = {" << join(ait->getSessionDirectory(), ", ") << "}"<<endl;
	cout << "STAGEOUT Dir   = {" << join(ait->getStageOutDirectory(), ", ") <<"}"<< endl;
	
      }
    
    delete call;
    
    //return 0;
    //}
  return 0;
}
