/**
   build with:

   g++ -Wall -g -O0 -I../build/src/ES -I../interface -o service service.cpp ../build/src/ES/autogen/ES_CLIENTServer.cpp -L../../stage/usr/lib64 -lglite_ce_es_client -lgridsite -lgsoap++ -lglite_security_gsoap_plugin_2713_cxx
*/
#include "autogen/ES_CLIENTStub.h"
#include "autogen/ES_CLIENT.nsmap"

#include "glite/ce/es-client-api-c/ActivityDescription.h"
#include "glite/ce/es-client-api-c/ActivityCreationResponse.h"
#include "glite/ce/es-client-api-c/ListActivities.h"
#include "glite/ce/es-client-api-c/ComputingActivity.h"
#include "glite/ce/es-client-api-c/ComputingActivityAssociations.h"

#include "glite/ce/es-client-api-c/util.h"

#include <boost/lexical_cast.hpp>
#include <boost/algorithm/string/split.hpp>
#include <boost/algorithm/string.hpp>
#include <boost/tuple/tuple.hpp>

#include <iostream>
#include <sys/time.h>
#include <cstdlib>
#include <set>
#include <map>

SOAP_NMAC struct Namespace namespaces[] = {};

extern "C" {
#undef IOV_MAX // this is very ugly, but the only way to silent a warning concerning
               // the redefinition of IOV_MAX
#include "glite/security/glite_gsplugin.h"      
}

using namespace std;
using namespace emi_es::client::wrapper;
using namespace emi_es::client::util;

map<string, ActivityStatus::ACTIVITYSTATUS> jobs;
map<string, boost::tuple<time_t, string, string> > delegs;

char* notifyMessage[] = { "CLIENT_DATAPULL_DONE", "CLIENT_DATAPUSH_DONE"};

string getPeerName( int socket );

/**
 *
 *
 *
 *
 *
 *
 */
string resolveName(const string& address) {
  struct addrinfo * result;
  struct addrinfo * res;
  int error;
  
  /* resolve the domain name into a list of addresses */
  error = getaddrinfo(address.c_str(), NULL, NULL, &result);
  
  if (0 != error)
    {   
      perror("error in getaddrinfo: ");
      return "UnresolvedHost";
    }   
  
  if (result == NULL)
    {   
      perror("getaddrinfo found no results\n");
      return "UnresolvedHost";
    }   
  
  for (res = result; res != NULL; res = res->ai_next)
    {   
      char hostname[NI_MAXHOST] = "";
      
      error = getnameinfo(res->ai_addr, res->ai_addrlen, hostname, NI_MAXHOST, NULL, 0, 0); 
      
      if (0 != error)
        {
          fprintf(stderr, "error in getnameinfo: %s\n", gai_strerror(error));
          continue;
        }
      
      if (*hostname)
        {
          printf("hostname: %s\n", hostname);
	  return hostname;
        }
      
    }   
  
  freeaddrinfo(result);
  return "";
}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESCREATION__CreateActivity( struct soap* SOAP, 
			      _ESCREATIONTYPES__CreateActivities *CA, 
			      _ESCREATIONTYPES__CreateActivitiesResponse *CAresp )
{
  
  cout << "\n--------------------------------------\nCalled method service::CreateActivity" << endl;
  
  if( CA->ESADL__ActivityDescription.begin( ) 
      == CA->ESADL__ActivityDescription.end() ) {
    
    InternalBaseFault* fault = new InternalBaseFault( "No activity defined by the client", 
						       time(0), 
						       "Fault caused by wrong client's request", 
						       2);

    soap_receiver_fault(SOAP, "Stack dump", NULL);
    
    if (SOAP->version == 2)
      {
	SOAP->fault->SOAP_ENV__Detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
	SOAP->fault->SOAP_ENV__Detail->__type = SOAP_TYPE__ESCREATIONTYPES__InvalidActivityDescriptionFault;
	SOAP->fault->SOAP_ENV__Detail->fault = fault; // point to stack
	SOAP->fault->SOAP_ENV__Detail->__any = NULL; // no other XML data
      }
    else 
      {
	SOAP->fault->detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
	SOAP->fault->detail->__type = SOAP_TYPE__ESCREATIONTYPES__InvalidActivityDescriptionFault;
	SOAP->fault->detail->fault = fault; // point to stack
	SOAP->fault->detail->__any = NULL; // no other XML data
      }
    return SOAP_CLI_FAULT;
  }

  if(::getenv("ACTDESCFAULT")) {
    _ESCREATIONTYPES__InvalidActivityDescriptionFault* fault = new _ESCREATIONTYPES__InvalidActivityDescriptionFault();
    fault->Message = "FAULT ActDesc for this Activity";
    fault->Timestamp = new time_t(time(0));
    fault->Description = new string("Fault ActDesc requested by client");
    fault->FailureCode = new int(149);
    soap_receiver_fault(SOAP, "Fault String set by server", NULL);
    if (SOAP->version == 2)
      {
	SOAP->fault->SOAP_ENV__Detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
	SOAP->fault->SOAP_ENV__Detail->__type = SOAP_TYPE__ESCREATIONTYPES__InvalidActivityDescriptionFault;
	SOAP->fault->SOAP_ENV__Detail->fault = fault; // point to stack
	SOAP->fault->SOAP_ENV__Detail->__any = NULL; // no other XML data
      }
    else 
      {
	SOAP->fault->detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
	SOAP->fault->detail->__type = SOAP_TYPE__ESCREATIONTYPES__InvalidActivityDescriptionFault;
	SOAP->fault->detail->fault = fault; // point to stack
	SOAP->fault->detail->__any = NULL; // no other XML data
      }
    return SOAP_CLI_FAULT;
  }

  vector<_ESADL__ActivityDescription*>::const_iterator it;

  bool insert = true;

  for(it = CA->ESADL__ActivityDescription.begin( );
      it != CA->ESADL__ActivityDescription.end( );
      ++it )
    {
      ActivityDescription *adesc = (ActivityDescription *)*it;
      cout << *adesc->ESADL__ActivityIdentification->Name << endl;

      InternalBaseFault *fault = 0;
      insert = true;
      vector<string> pieces;
//       if((*adesc->ESADL__ActivityIdentification->Name).find("fault")!= string::npos)
// 	{
// 	  boost::split( pieces, *adesc->ESADL__ActivityIdentification->Name, boost::is_any_of(":"));
// 	  if(pieces[1]=="int")
// 	    fault = new InternalBaseFault("FAULT for this Activity", 
// 					  time(0), 
// 					  "Fault requested by client", 
// 					  2);
// 	  if(pieces[1]=="vec") {
// 	    fault = new _ESTYPES__VectorLimitExceededFault();
// 	    ((_ESTYPES__VectorLimitExceededFault*)fault)->ServerLimit = 123;
// 	    fault->Message = "FAULT Vector for this Activity";
// 	    fault->Timestamp = new time_t(time(0));
// 	    fault->Description = new string("Fault Vector requested by client");
// 	    fault->FailureCode = new int(149);
// 	  }

// 	  if(pieces[1]=="desc") {
// 	    fault = new _ESCREATIONTYPES__InvalidActivityDescriptionFault();
// 	    fault->Message = "FAULT ActDesc for this Activity";
// 	    fault->Timestamp = new time_t(time(0));
// 	    fault->Description = new string("Fault ActDesc requested by client");
// 	    fault->FailureCode = new int(149);
// 	  }
	  
// 	  insert = false;
// 	}
      
      DirectoryReferenceType ref1,ref2,ref3;
      for(int i = 0; i < GetRandom::get_random(4); ++i )
	ref1.URL.push_back("http://URL-" + boost::lexical_cast<string>(GetRandom::get_random(100)) + ":8080/MyPath");
      
      for(int i = 0; i < GetRandom::get_random(4); ++i )
	ref2.URL.push_back("http://URL-" + boost::lexical_cast<string>(GetRandom::get_random(100)) + ":8080/MyPath");
      
      for(int i = 0; i < GetRandom::get_random(4); ++i )
	ref3.URL.push_back("http://URL-" + boost::lexical_cast<string>(GetRandom::get_random(100)) + ":8080/MyPath");
      
      
      vector<int> attrs;
      for(int i = 0; i < GetRandom::get_random(10); ++i )
	attrs.push_back( GetRandom::get_random(18) );
      
      ActivityStatus *status = new ActivityStatus( GetRandom::get_random(7), attrs, time(0), "The Activity has been created!");
      
      string job = string("JobID_EMIES_") + boost::lexical_cast<string>(GetRandom::get_random(946725));
      if(insert)
	jobs.insert( make_pair(job, status->getStatusNum() ) );
      
      ActivityCreationResponse* aresp = new ActivityCreationResponse( fault,
								      job,
								      "lxgrid05",
								      *status,
								      time(0),
								      ref1,
								      ref2,
								      ref3
								      );
      
      
      CAresp->ESCREATIONTYPES__ActivityCreationResponse.push_back( aresp );
    }

  return SOAP_OK;

}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5
int
SOAP_FMAC6 
__ESACTIVITY__ListActivities
(soap* SOAP, _ESACTIVITYTYPES__ListActivities* list, _ESACTIVITYTYPES__ListActivitiesResponse* resp) 
{ 
  cout << "\n\n--------------------------------------\nCalled method service::ListActivity" << endl;
  ListActivities *List = (ListActivities *)list;
  cout << List->toString( ) << endl;

  resp->truncated = false;

  set<ActivityStatus::ACTIVITYSTATUS> States;
  {
    vector<ESTYPES__PrimaryActivityStatus>::const_iterator it = list->Status.begin();
    for( ; it != list->Status.end( ); ++it )
      States.insert( (ActivityStatus::ACTIVITYSTATUS)*it );
  }

  map<string, ActivityStatus::ACTIVITYSTATUS>::const_iterator it = jobs.begin( );
  for( ; it != jobs.end( ); ++it )
  {
    if( States.find( it->second ) != States.end() )
      resp->ESTYPES__ActivityID.push_back( it->first );
  }

  return SOAP_OK; 
}


/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5
int
SOAP_FMAC6
__ESACTIVITYMANAGEMENT__GetActivityStatus
(soap*, _ESACTIVITYTYPES__GetActivityStatus*, _ESACTIVITYTYPES__GetActivityStatusResponse*) 
{return SOAP_OK;}

SOAP_FMAC5
int
SOAP_FMAC6
__ES_USCOREACTIVITY__GetActivityStatus(soap* SOAP, 
				       _ESACTIVITYTYPES__GetActivityStatus* req, 
				       _ESACTIVITYTYPES__GetActivityStatusResponse* resp) 
{
  //cout << "GetActivityStatus CALLED !" << endl;
  cout << "\n\n--------------------------------------\nCalled method service::GetActivityStatus" << endl;
  vector<string>::const_iterator it = req->ESTYPES__ActivityID.begin( );
  for( ; it != req->ESTYPES__ActivityID.end( ); ++it )
  {
    ActivityStatusItemType* item = new ActivityStatusItemType();

    cout << "Handling status for job " << *it << endl;

    if(jobs.find( *it ) != jobs.end( ) ) {

      item->ESTYPES__ActivityID = *it;
      item->__union_ActivityStatusItem = SOAP_UNION__ESACTIVITYTYPES__union_ActivityStatusItem_ESTYPES__ActivityStatus;

      vector<int> attrs;
      for(int jj = 0; jj<GetRandom::get_random(9); ++jj)
	attrs.push_back( GetRandom::get_random(17) );

      item->union_ActivityStatusItem.ESTYPES__ActivityStatus 
	= ((_ESTYPES__ActivityStatus*)new ActivityStatus(GetRandom::get_random(7), attrs, time(0), "Description"));
      resp->ESACTIVITYTYPES__ActivityStatusItem.push_back( item );

      

    } else {
      // SEND FAULT !!
      InternalBaseFault *F = new InternalBaseFault("Not existing job "+*it, time(0), "Job not handled by this system", 1 );
      item->ESTYPES__ActivityID = *it;
      item->__union_ActivityStatusItem = SOAP_UNION__ESACTIVITYTYPES__union_ActivityStatusItem_ESTYPES__InternalBaseFault;
      item->union_ActivityStatusItem.ESTYPES__InternalBaseFault = F;
      resp->ESACTIVITYTYPES__ActivityStatusItem.push_back( item );
    }
  }
  return SOAP_OK; 
}




/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESACTIVITYMANAGEMENT__GetActivityInfo(soap* SOAP, 
				     _ESACTIVITYTYPES__GetActivityInfo* _info, 
				     _ESACTIVITYTYPES__GetActivityInfoResponse* resp) 
{ 
  cout << "\n\n--------------------------------------\nCalled method service::GetActivityInfo" << endl;
  _ESACTIVITYTYPES__ActivityInfoItem *info = new _ESACTIVITYTYPES__ActivityInfoItem();
  info->ESTYPES__ActivityID = "ALVISE";
  info->__union_ActivityInfoItem = SOAP_UNION__ESACTIVITYTYPES__union_ActivityInfoItem_ESTYPES__ActivityInfo;
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo = new ns1__ComputingActivity_USCOREt();
  
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo->Type = new ns1__ComputingActivityType_USCOREt((ns1__ComputingActivityType_USCOREt)1);
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo->IDFromEndpoint = new string( "IDFromEndpoint" );
  resp->ESACTIVITYTYPES__ActivityInfoItem.push_back(info);
  return SOAP_OK; 
}





/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESACTIVITYMANAGEMENT__PauseActivity(soap* SOAP, 
					     _ESACTIVITYMANAGEMENTTYPES__PauseActivity* PA, 
					     _ESACTIVITYMANAGEMENTTYPES__PauseActivityResponse* resp) 
{ 
//   cout << "\n\n--------------------------------------\nCalled method service::PauseActivity" << endl;
//   vector<string> requestedIDS = PA->ES_USCORETYPES__ActivityID;
// 
//   vector<string>::const_iterator it = requestedIDS.begin( );
// 
//   for( ; it != requestedIDS.end( ); ++it ) {
// 
//     if( jobs.find( *it ) == jobs.end( ) ) {
//       cerr << "Pause:: Job [" << *it << "] NOT FOUND !!" << endl;
//       _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//       respItem->ES_USCORETYPES__ActivityID = *it;
//       respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_ES_USCORETYPES__InternalBaseFault;
//       respItem->union_ResponseItem.ES_USCORETYPES__InternalBaseFault = new InternalBaseFault( string("Job ")+*it+" NOT FOUND", time(0), "Job not present in this ES", 4 );
//       resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//       continue;
//     }
//   
//     _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//     respItem->ES_USCORETYPES__ActivityID = *it;
//     respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_EstimatedTime;
//     respItem->union_ResponseItem.EstimatedTime = new ULONG64( time(0) + GetRandom::get_random(3600)); 
//     resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//     
//   }
  return SOAP_OK; 
}


/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESACTIVITYMANAGEMENT__ResumeActivity(soap* SOAP,
					      _ESACTIVITYMANAGEMENTTYPES__ResumeActivity* req,
					      _ESACTIVITYMANAGEMENTTYPES__ResumeActivityResponse* resp) 
{ 
//   cout << "\n\n--------------------------------------\nCalled method service::ResumeActivity" << endl;
//   vector<string> requestedIDS = req->ES_USCORETYPES__ActivityID;
// 
//   vector<string>::const_iterator it = requestedIDS.begin( );
// 
//   for( ; it != requestedIDS.end( ); ++it ) {
// 
//     if( jobs.find( *it ) == jobs.end( ) ) {
//       cerr << "Resume:: Job [" << *it << "] NOT FOUND !!" << endl;
//       _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//       respItem->ES_USCORETYPES__ActivityID = *it;
//       respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_ES_USCORETYPES__InternalBaseFault;
//       respItem->union_ResponseItem.ES_USCORETYPES__InternalBaseFault = new InternalBaseFault( string("Job ")+*it+" NOT FOUND", time(0), "Job not present in this ES", 4 );
//       resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//       continue;
//     }
//   
//     _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//     respItem->ES_USCORETYPES__ActivityID = *it;
//     respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_EstimatedTime;
//     respItem->union_ResponseItem.EstimatedTime = new ULONG64( time(0) + GetRandom::get_random(3600)); 
//     resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//     
//   }

  return SOAP_OK; 
}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESACTIVITYMANAGEMENT__CancelActivity(soap* SOAP, 
					      _ESACTIVITYMANAGEMENTTYPES__CancelActivity* req, 
					      _ESACTIVITYMANAGEMENTTYPES__CancelActivityResponse* resp) 
{ 
//   cout << "\n\n--------------------------------------\nCalled method service::CancelActivity" << endl;
//   vector<string> requestedIDS = req->ES_USCORETYPES__ActivityID;
//   
//   vector<string>::const_iterator it = requestedIDS.begin( );
//   
//   for( ; it != requestedIDS.end( ); ++it ) {
//     
//     if( jobs.find( *it ) == jobs.end( ) ) {
//       cerr << "Resume:: Job [" << *it << "] NOT FOUND !!" << endl;
//       _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//       respItem->ES_USCORETYPES__ActivityID = *it;
//       respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_ES_USCORETYPES__InternalBaseFault;
//       respItem->union_ResponseItem.ES_USCORETYPES__InternalBaseFault = new InternalBaseFault( string("Job ")+*it+" NOT FOUND", time(0), "Job not present in this ES", 4 );
//       resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//       continue;
//     }
//   
//     _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//     respItem->ES_USCORETYPES__ActivityID = *it;
//     respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_EstimatedTime;
//     respItem->union_ResponseItem.EstimatedTime = new ULONG64( time(0) + GetRandom::get_random(3600)); 
//     resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//     
//   }
  return SOAP_OK; 
}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESACTIVITYMANAGEMENT__WipeActivity(soap* SOAP, 
					    _ESACTIVITYMANAGEMENTTYPES__WipeActivity* req,
					    _ESACTIVITYMANAGEMENTTYPES__WipeActivityResponse* resp) 
{ 
//   cout << "\n\n--------------------------------------\nCalled method service::WipeActivity" << endl;
//   vector<string> requestedIDS = req->ES_USCORETYPES__ActivityID;
//   
//   vector<string>::const_iterator it = requestedIDS.begin( );
//   
//   for( ; it != requestedIDS.end( ); ++it ) {
//     
//     if( jobs.find( *it ) == jobs.end( ) ) {
//       cerr << "Resume:: Job [" << *it << "] NOT FOUND !!" << endl;
//       _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//       respItem->ES_USCORETYPES__ActivityID = *it;
//       respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_ES_USCORETYPES__InternalBaseFault;
//       respItem->union_ResponseItem.ES_USCORETYPES__InternalBaseFault = new InternalBaseFault( string("Job ")+*it+" NOT FOUND", time(0), "Job not present in this ES", 4 );
//       resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//       continue;
//     }
//   
//     _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//     respItem->ES_USCORETYPES__ActivityID = *it;
//     respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_EstimatedTime;
//     respItem->union_ResponseItem.EstimatedTime = new ULONG64( time(0) + GetRandom::get_random(3600)); 
//     resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//     
//   }
  return SOAP_OK; 
}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5
int 
SOAP_FMAC6
__ESACTIVITYMANAGEMENT__RestartActivity(soap* SOAP, 
					       _ESACTIVITYMANAGEMENTTYPES__RestartActivity* req, 
					       _ESACTIVITYMANAGEMENTTYPES__RestartActivityResponse* resp) 
{
//   cout << "\n\n--------------------------------------\nCalled method service::RestartActivity" << endl;
//   
//   vector<string> requestedIDS = req->ES_USCORETYPES__ActivityID;
//   
//   vector<string>::const_iterator it = requestedIDS.begin( );
//   
//   for( ; it != requestedIDS.end( ); ++it ) {
//     
//     if( jobs.find( *it ) == jobs.end( ) ) {
//       cerr << "Resume:: Job [" << *it << "] NOT FOUND !!" << endl;
//       _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//       respItem->ES_USCORETYPES__ActivityID = *it;
//       respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_ES_USCORETYPES__InternalBaseFault;
//       respItem->union_ResponseItem.ES_USCORETYPES__InternalBaseFault = new InternalBaseFault( string("Job ")+*it+" NOT FOUND", time(0), "Job not present in this ES", 4 );
//       resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//       continue;
//     }
//   
//     _ES_USCOREACTIVITYMANAGEMENT__ResponseItem *respItem = new _ES_USCOREACTIVITYMANAGEMENT__ResponseItem();
//     respItem->ES_USCORETYPES__ActivityID = *it;
//     respItem->__union_ResponseItem = SOAP_UNION__ES_USCOREACTIVITYMANAGEMENT__union_ResponseItem_EstimatedTime;
//     respItem->union_ResponseItem.EstimatedTime = new ULONG64( time(0) + GetRandom::get_random(3600)); 
//     resp->ES_USCOREACTIVITYMANAGEMENT__ResponseItem.push_back( respItem );
//     
//   }
  return SOAP_OK; 
}


/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESACTIVITYMANAGEMENT__NotifyService(soap* SOAP, 
					     _ESACTIVITYMANAGEMENTTYPES__NotifyService* req,
					     _ESACTIVITYMANAGEMENTTYPES__NotifyServiceResponse* resp) 
{ 
//   cout << "\n\n--------------------------------------\nCalled method service::NotifyService" << endl;
// 
//   vector<_ES_USCOREACTIVITYMANAGEMENT__NotifyRequestItem*>::const_iterator it;
//   for( it = req->ES_USCOREACTIVITYMANAGEMENT__NotifyRequestItem.begin( );
//        it != req->ES_USCOREACTIVITYMANAGEMENT__NotifyRequestItem.end( );
//        ++it )
//     {
//       cout << "ID=" << (*it)->ES_USCORETYPES__ActivityID << endl;
// 
//       if( jobs.find( (*it)->ES_USCORETYPES__ActivityID ) == jobs.end( ) ) {
// 	cerr << "Not found job " <<  (*it)->ES_USCORETYPES__ActivityID << endl;
// 	_ES_USCOREACTIVITYMANAGEMENT__NotifyResponseItem* item = new _ES_USCOREACTIVITYMANAGEMENT__NotifyResponseItem();
// 	item->ES_USCORETYPES__ActivityID = (*it)->ES_USCORETYPES__ActivityID;
// 	item->ES_USCORETYPES__InternalBaseFault = new InternalBaseFault(string("Job ")+(*it)->ES_USCORETYPES__ActivityID+" NOT FOUND", time(0), "Job not present in this ES", 4 );
// 	resp->ES_USCOREACTIVITYMANAGEMENT__NotifyResponseItem.push_back( item );
// 	continue;
//       }
// 
//       if( (*it)->NotifyMessage == 0 || (*it)->NotifyMessage==1 )
// 	cout << (*it)->ES_USCORETYPES__ActivityID << "  -->  " << notifyMessage[(*it)->NotifyMessage] << endl;
//       else
// 	cout << (*it)->ES_USCORETYPES__ActivityID << "  -->  Unrecognized client's message" << endl;
// 
//       _ES_USCOREACTIVITYMANAGEMENT__NotifyResponseItem *item = new _ES_USCOREACTIVITYMANAGEMENT__NotifyResponseItem();
//       item->ES_USCORETYPES__ActivityID = (*it)->ES_USCORETYPES__ActivityID;
//       item->ES_USCORETYPES__InternalBaseFault = 0;
//       resp->ES_USCOREACTIVITYMANAGEMENT__NotifyResponseItem.push_back( item );
//     }
  return SOAP_OK; 
}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESRESOURCEINFO__GetResourceInfo
(soap* SOAP, _ESRESOURCEINFOTYPES__GetResourceInfo*, _ESRESOURCEINFOTYPES__GetResourceInfoResponse* resp) 
{ 
  
//   {
//     resp->ComputingService = new OGF_USCOREGLUE2SCHEMA__ComputingService_USCOREt();
//     {
//       resp->ComputingService->TotalJobs          = new unsigned int( GetRandom::get_random(1000) );
//       resp->ComputingService->RunningJobs        = new unsigned int( GetRandom::get_random(1000) );
//       resp->ComputingService->WaitingJobs        = new unsigned int( GetRandom::get_random(1000) ); 
//       resp->ComputingService->StagingJobs        = new unsigned int( GetRandom::get_random(1000) );
//       resp->ComputingService->SuspendedJobs      = new unsigned int( GetRandom::get_random(1000) );
//       resp->ComputingService->PreLRMSWaitingJobs = new unsigned int( GetRandom::get_random(1000) );
//       {
// 	OGF_USCOREGLUE2SCHEMA__ComputingEndpoint_USCOREt* end1;
// 	{
// 	  end1 = new OGF_USCOREGLUE2SCHEMA__ComputingEndpoint_USCOREt();
// 	  end1->Staging = new string( "/var/somepath1" );
// 	  end1->JobDescription.push_back("desc1");
// 	  end1->JobDescription.push_back("desc2");
// 	  end1->TotalJobs          = new unsigned int( GetRandom::get_random(1000) );
// 	  end1->RunningJobs        = new unsigned int( GetRandom::get_random(1000) );
// 	  end1->WaitingJobs        = new unsigned int( GetRandom::get_random(1000) ); 
// 	  end1->StagingJobs        = new unsigned int( GetRandom::get_random(1000) );
// 	  end1->SuspendedJobs      = new unsigned int( GetRandom::get_random(1000) );
// 	  end1->PreLRMSWaitingJobs = new unsigned int( GetRandom::get_random(1000) );
// 	  {
// 	    end1->Associations       = new _OGF_USCOREGLUE2SCHEMA__ComputingEndpoint_USCOREt_Associations();
// 	    end1->Associations->ComputingShareID.push_back("PIPO1");
// 	    end1->Associations->ComputingShareID.push_back("PIPO2");
// 	  }
// 	  {
// 	    end1->ComputingActivities= new  _OGF_USCOREGLUE2SCHEMA__ComputingEndpoint_USCOREt_ComputingActivities();
// 	    set<string>::const_iterator it = jobs.begin();
// 	    ComputingActivityAssociations ass("User's domain",
// 					      "cream-52",
// 					      "shareID",
// 					      "exeenvID",
// 					      vector<string>());
// 	    for( ; it!=jobs.end(); ++it) {
// 	      OGF_USCOREGLUE2SCHEMA__ComputingActivity_USCOREt *act1 
// 		= new ComputingActivity( "Job MPI",
// 					 *it,
// 					 *it+"-LRMS",
// 					 "Job description",
// 					 vector<string>(),
// 					 vector<string>(),
// 					 0,
// 					 "1",
// 					 vector<string>(),
// 					 2,
// 					 "User's domain",
// 					 "Alvise Dorigo",
// 					 "glite001",
// 					 120,
// 					 100,
// 					 2,
// 					 vector<string>(),
// 					 "input",
// 					 "output",
// 					 "error",
// 					 "/var/log",
// 					 vector<string>(),
// 					 "medium",
// 					 100,
// 					 100,
// 					 1024,
// 					 time(0)-120,
// 					 time(0)-60,
// 					 time(0)-50,
// 					 time(0)-10,
// 					 time(0)-5,
// 					 time(0),
// 					 time(0)+1800,
// 					 "lxgrid05.pd.infn.it",
// 					 "mac-grid-01.pd.infn.it",
// 					 vector<string>(),
// 					 ass);
	      
// 	      end1->ComputingActivities->ComputingActivity.push_back(act1);
// 	    }
// 	  }
// 	  end1->URL = "https://casamia.cittamia";
// 	  end1->Capability.push_back( 0 );
// 	  end1->Capability.push_back( 1 );
// 	  end1->Technology = new string("Technology");
// 	  end1->InterfaceName = "EMIES";
// 	  end1->InterfaceVersion.push_back( "v1.0" );
// 	  end1->InterfaceVersion.push_back( "v1.1" );
// 	  end1->InterfaceExtension.push_back("4.0.5");
// 	  end1->WSDL.push_back( "WSDL" );
// 	  end1->SupportedProfile.push_back("profile");
// 	  end1->Semantics.push_back("sematiche");
// 	  end1->Implementor = new string("IO");
// 	  end1->ImplementationName = new string("CREAM");
// 	  end1->ImplementationVersion = new string("v1.0");
// 	  end1->QualityLevel = OGF_USCOREGLUE2SCHEMA__QualityLevel_USCOREt__development;
// 	  end1->HealthState = OGF_USCOREGLUE2SCHEMA__EndpointHealthState_USCOREt__unknown;
// 	  end1->HealthStateInfo = new string("crashato");
// 	  end1->ServingState = OGF_USCOREGLUE2SCHEMA__ServingState_USCOREt__draining;
// 	  end1->StartTime = new time_t(time(0)-86400);
// 	  end1->IssuerCA = new string("mestesso");
// 	  end1->TrustedCA.push_back("INFN");
// 	  end1->TrustedCA.push_back("CERN");
// 	  end1->DowntimeAnnounce = new time_t(time(0));
// 	  end1->DowntimeStart = new time_t(time(0));
// 	  end1->DowntimeEnd = new time_t( time(0));
// 	  end1->DowntimeInfo = new string("BAU!");
// 	  {
// 	    end1->AccessPolicy = new OGF_USCOREGLUE2SCHEMA__AccessPolicy_USCOREt();
// 	    end1->AccessPolicy->Scheme = "Schema";
// 	    end1->AccessPolicy->Rule.push_back("Regola1");
// 	    end1->AccessPolicy->Rule.push_back("Regola2");
// 	    end1->AccessPolicy->BaseType = "BASEPOLICY";
// 	    {
// 	      end1->AccessPolicy->Associations = new _OGF_USCOREGLUE2SCHEMA__Policy_USCOREt_Associations();
// 	      end1->AccessPolicy->Associations->UserDomain.push_back("USERDOMAIN1");
// 	      end1->AccessPolicy->Associations->UserDomain.push_back("USERDOMAIN2");
// 	    }
// 	  }
// 	  end1->BaseType = "BASE";
// 	  // Entity
// 	  end1->ID = "ID";
// 	  end1->Name = new string("Name");
// 	  end1->OtherInfo.push_back("Fuck gently");
// 	  end1->Extensions = new Extensions( vector<OGF_USCOREGLUE2SCHEMA__Extension_USCOREt>() );
// 	  end1->CreationTime = new time_t( time(0));
// 	  end1->Validity = new ULONG64( 3598634 );
// 	}

// 	resp->ComputingService->ComputingEndpoint.push_back(end1);
// 	//resp->ComputingService->ComputingEndpoint.push_back(end2);
// 	//resp->ComputingService->ComputingEndpoint.push_back(end3);
//       }
//     }
//   }
  
//   {
//     resp->ActivityManager = new OGF_USCOREGLUE2SCHEMA__Service_USCOREt();
    
//   }
  
   return SOAP_OK; 
 }

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC3 
void 
SOAP_FMAC4 
soap_serialize_xsd__QName(struct soap*, const xsd__QName *) { }

SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESRESOURCEINFO__QueryResourceInfo
(soap*, _ESRESOURCEINFOTYPES__QueryResourceInfo*, ns1__Entity_USCOREt*) 
{ return SOAP_OK; }

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5
int
SOAP_FMAC6 
__ESDELEGATION__GetDelegationInfo
(soap* SOAP, 
 _ESDELEGATIONTYPES__GetDelegationInfo* req, 
 _ESDELEGATIONTYPES__GetDelegationInfoResponse* resp) 
{ 
  cout << "GetDelegationInfo CALLED!" << endl;
  
  string ID = req->DelegationID;

  map<string, boost::tuple<time_t,string,string> >::iterator info = delegs.find(req->DelegationID);
//   if(info == delegs.end())
//     {
//       _ES_USCOREDELEGATION__UnknownDelegationIDFault *f = new _ES_USCOREDELEGATION__UnknownDelegationIDFault();
//       f->Message = "Delegation not found";
//       f->Timestamp = new time_t( time(0) );
//       f->Description = new string( "Client requested info for an unknown delegation ID [" + ID + "]" );
//       f->FailureCode = new int(14);
      
//       soap_receiver_fault(SOAP, "Stack dump", NULL);
      
//       if (SOAP->version == 2)
// 	{
// 	  SOAP->fault->SOAP_ENV__Detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
// 	  SOAP->fault->SOAP_ENV__Detail->__type = SOAP_TYPE__ES_USCOREDELEGATION__UnknownDelegationIDFault;
// 	  SOAP->fault->SOAP_ENV__Detail->fault = f; // point to stack
// 	  SOAP->fault->SOAP_ENV__Detail->__any = NULL; // no other XML data
// 	} else {
// 	SOAP->fault->detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
// 	SOAP->fault->detail->__type = SOAP_TYPE__ES_USCOREDELEGATION__UnknownDelegationIDFault;
// 	SOAP->fault->detail->fault = f; // point to stack
// 	SOAP->fault->detail->__any = NULL; // no other XML data
//       }
//       return SOAP_CLI_FAULT;
//     }
  
  resp->Lifetime = info->second.get<0>();
  resp->Issuer = new string( info->second.get<1>());
  resp->Subject = new string( info->second.get<2>());
  return SOAP_OK; 
}


/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5
int
SOAP_FMAC6
__ESDELEGATION__PutDelegation(soap* SOAP, 
				     _ESDELEGATIONTYPES__PutDelegation* req, 
				     std::basic_string<char, std::char_traits<char>, std::allocator<char> >& resp) 
{ 
//   cout << "PutDelegation CALLED!" << endl<<endl;
//   cout << "DelegationID=" << req->DelegationID << endl;
//   cout << "Credential  =" << req->Credential << endl;
//   resp = "Delegation ACCEPTED";
  return SOAP_OK;
}

/**
 *
 *
 *
 *
 *
 *
 */
SOAP_FMAC5 
int 
SOAP_FMAC6 
__ESDELEGATION__InitDelegation(soap* SOAP, 
				      _ESDELEGATIONTYPES__InitDelegation* init, 
				      _ESDELEGATIONTYPES__InitDelegationResponse* resp) 
{ 
//   cout << "InitDelegation CALLED!" << endl;
// 
//   string id = init->CredentialType;
//   if(id.find("fault") != string::npos ) {
//     _ES_USCOREDELEGATION__InternalServiceDelegationFault* fault = new _ES_USCOREDELEGATION__InternalServiceDelegationFault( );
//     fault->Message = "Delegation Error requested by client";
//     fault->Timestamp = new time_t( time(0) );
//     fault->Description = new string( "Fault caused by client's request" );
//     fault->FailureCode = new int(14);
//     
//     soap_receiver_fault(SOAP, "Stack dump", NULL);
//     
//     if (SOAP->version == 2)
//       {
// 	SOAP->fault->SOAP_ENV__Detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
// 	SOAP->fault->SOAP_ENV__Detail->__type = SOAP_TYPE__ES_USCOREDELEGATION__InternalServiceDelegationFault;
// 	SOAP->fault->SOAP_ENV__Detail->fault = fault; // point to stack
// 	SOAP->fault->SOAP_ENV__Detail->__any = NULL; // no other XML data
//       }
//     else 
//       {
// 	SOAP->fault->detail = (struct SOAP_ENV__Detail*)soap_malloc(SOAP, sizeof(struct SOAP_ENV__Detail));
// 	SOAP->fault->detail->__type = SOAP_TYPE__ES_USCOREDELEGATION__InternalServiceDelegationFault;
// 	SOAP->fault->detail->fault = fault; // point to stack
// 	SOAP->fault->detail->__any = NULL; // no other XML data
//       }
//     return SOAP_CLI_FAULT;
//   }
// 
//   resp->DelegationID = string("ID-") + boost::lexical_cast<string>(GetRandom::get_random(1000000));
//   resp->CSR = new string( "Credenziale da firmare" );
// 
//   boost::tuple<time_t,string,string> delegInfo = boost::make_tuple( time(0)+GetRandom::get_random(7200), "issuer", "Subject");
// 
//   delegs[resp->DelegationID] = delegInfo;

  return SOAP_OK; 
}





SOAP_FMAC5 
int 
SOAP_FMAC6
__ESACTIVITY__GetActivityInfo
(soap*, _ESACTIVITYTYPES__GetActivityInfo*, _ESACTIVITYTYPES__GetActivityInfoResponse* resp) 
{ 
  cout << "\n\n--------------------------------------\nCalled method service::GetActivityInfo" << endl;
  _ESACTIVITYTYPES__ActivityInfoItem *info = new _ESACTIVITYTYPES__ActivityInfoItem();
  info->ESTYPES__ActivityID = "ALVISE";
  info->__union_ActivityInfoItem = SOAP_UNION__ESACTIVITYTYPES__union_ActivityInfoItem_ESTYPES__ActivityInfo;
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo = new ns1__ComputingActivity_USCOREt();
  
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo->Type = new ns1__ComputingActivityType_USCOREt((ns1__ComputingActivityType_USCOREt)1);
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo->IDFromEndpoint = new string( "IDFromEndpoint" );
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo->Extensions = new ns1__Extensions_USCOREt();
  ns1__Extension_USCOREt *ext = new ns1__Extension_USCOREt();
  ext->LocalID = "LocalID";
  ext->Key = "Key";
  ext->Value = "Value";
  info->union_ActivityInfoItem.ESTYPES__ActivityInfo->Extensions->Extension.push_back( ext );
  
  resp->ESACTIVITYTYPES__ActivityInfoItem.push_back(info);
  
  return SOAP_OK; 
}


SOAP_FMAC5 
int 
SOAP_FMAC6
__ESACTIVITY__GetActivityStatus
(soap*, _ESACTIVITYTYPES__GetActivityStatus* req, _ESACTIVITYTYPES__GetActivityStatusResponse* resp)
{ 
  cout << "\n\n--------------------------------------\nCalled method service::GetActivityStatus" << endl;
  vector<string>::const_iterator it = req->ESTYPES__ActivityID.begin( );
  for( ; it != req->ESTYPES__ActivityID.end( ); ++it )
  {
    ActivityStatusItemType* item = new ActivityStatusItemType();

    cout << "Handling status for job " << *it << endl;

    if(jobs.find( *it ) != jobs.end( ) ) {

      item->ESTYPES__ActivityID = *it;
      item->__union_ActivityStatusItem = SOAP_UNION__ESACTIVITYTYPES__union_ActivityStatusItem_ESTYPES__ActivityStatus;

      vector<int> attrs;
      for(int jj = 0; jj<GetRandom::get_random(9); ++jj)
        attrs.push_back( GetRandom::get_random(17) );

      item->union_ActivityStatusItem.ESTYPES__ActivityStatus
        = ((_ESTYPES__ActivityStatus*)new ActivityStatus(GetRandom::get_random(7), attrs, time(0), "Description"));
      resp->ESACTIVITYTYPES__ActivityStatusItem.push_back( item );



    } else {
      // SEND FAULT !!
      InternalBaseFault *F = new InternalBaseFault("Not existing job "+*it, time(0), "Job not handled by this system", 1 );
      item->ESTYPES__ActivityID = *it;
      item->__union_ActivityStatusItem = SOAP_UNION__ESACTIVITYTYPES__union_ActivityStatusItem_ESTYPES__InternalBaseFault;
      item->union_ActivityStatusItem.ESTYPES__InternalBaseFault = F;
      resp->ESACTIVITYTYPES__ActivityStatusItem.push_back( item );
    }
  }

  return SOAP_OK; 
}

/**
 *
 *
 *
 *
 *
 *
 */
int main( int argc, char* argv[] )
{
  if(argc<2) return 0;
  
  struct soap *soap;
  int m, s; 
  
  soap = soap_new();
  if(!soap) {
    fprintf(stderr, "Couldn't create SOAP structure. STOP!");
    exit(1);
  }
  
  glite_gsplugin_Context        ctx;
  
  if ( !::getenv("NOAUTH") && argv[2] && argv[3] ) {
    if ( glite_gsplugin_init_context(&ctx) ) {
      perror("init context"); 
      exit(1); 
    }
/*    ::unsetenv("X509_USER_CERT");
    ::unsetenv("X509_USER_KEY");
    ::setenv("X509_USER_CERT", argv[2], 0);
    ::setenv("X509_USER_KEY", argv[3], 0);*/
    if (glite_gsplugin_set_credential(ctx, argv[2], argv[3])) {
      fprintf (stderr, "Failed to set credentials. Maybe you need to remove the passphrase from the key pem file.\n");
      exit(1);
    }
  }
  
  if( !::getenv("NOAUTH") ) {
    if ( soap_register_plugin_arg(soap, glite_gsplugin, ctx? : NULL) ) {
      fprintf(stderr, "Can't register plugin\n");
      exit(1);
    }
  }

  soap_set_namespaces(soap, ES_CLIENT_namespaces);
  
  m = soap_bind(soap, NULL, atoi(argv[1]), 100);
  
  if (m < 0) {
    cerr << "Bind failed" << endl;
    soap_print_fault(soap, stderr);

  }
  else
    {
      for (int i = 1; ; i++)
        {
          s = soap_accept(soap);
          if (s < 0)
            {
	      cerr << "Accept failed" << endl;
              soap_print_fault(soap, stderr);
              break;
            }
          
          string buf = getPeerName( s );
          fprintf(stderr, 
                  "%d: accepted connection from IP=%s (%s) socket=%d\n",
                  i,
                  buf.c_str(),
                  resolveName(buf).c_str(),
                  s);

	  if(getenv("GENERATE_SOCKET_TIMEOUT"))
	    sleep(120);
        
          if(ES_CLIENT_serve(soap) != SOAP_OK) // process RPC request
            {
	      cerr << "ES_CLIENT_serve Failed !" << endl;
              soap_print_fault(soap, stderr); // print error
            }
          else {
	      fprintf(stderr, "\nrequest served\n");
	  }
          
          soap_destroy(soap); // clean up class instances
          soap_end(soap); // clean up everything and close socket
        }
    }
  soap_done(soap); // close master socket and detach environment 
}




/**
 *
 *
 *
 *
 *
 *
 *
 *
 */
string getPeerName( int socket ) {
  
    sockaddr_storage addr_client;
    int slen = sizeof(addr_client);
    getpeername(socket, (struct sockaddr *)&addr_client, (socklen_t*)&slen );
    
    char ipstr[INET6_ADDRSTRLEN];
    int port;
    
    if(addr_client.ss_family == AF_INET) {
      struct sockaddr_in *s = (struct sockaddr_in *)&addr_client;
      port = ntohs(s->sin_port);
      inet_ntop(AF_INET, &s->sin_addr, ipstr, sizeof ipstr);
    }
    
    if(addr_client.ss_family == AF_INET6) {
    
      struct sockaddr_in6 *s = (struct sockaddr_in6 *)&addr_client;
      port = ntohs(s->sin6_port);
      inet_ntop(AF_INET6, &s->sin6_addr, ipstr, sizeof ipstr);
    
    }
    
    string peer = ipstr;
      string::size_type pos = peer.find_last_of(':', 0);
      if(pos != string::npos ) {
        vector<string> SplitVec; // #2: Search for tokens
        boost::split( SplitVec, peer, boost::is_any_of(":"), boost::token_compress_on );
        peer = SplitVec.at( SplitVec.size()-1 );
      }
    
    return peer;
}
