/**
   build with:

  g++ -Wall -g -O0 -I../build/src/ES -I../interface -o statusActivity statusActivity.cpp -L../../stage/usr/lib64 -lglite_ce_es_client -lgridsite -lgsoap++ -lglite_security_gsoap_plugin_2713_cxx
  
*/
#include "glite/ce/es-client-api-c/GetActivityStatusResponse.h"
#include "glite/ce/es-client-api-c/GetActivityStatusCall.h"
#include "glite/ce/es-client-api-c/InternalBaseFault.h"
#include "glite/ce/es-client-api-c/CallFactory.h"
#include "glite/ce/es-client-api-c/util.h"

#include <string>
#include <vector>
#include <iostream>

using namespace std;
using namespace emi_es::client::wrapper;
using namespace emi_es::client::comm;
using namespace emi_es::client::util;

SOAP_NMAC struct Namespace namespaces[] = {};

void soap_serialize_xsd__QName(soap*, std::basic_string<char, std::char_traits<char>, std::allocator<char> > const*) {}

int main ( int argc, char* argv[] ) {
  if ( argc < 2 )
    return 0;
  
  string endpoint = argv[1];

  vector<string> states;
  for( int j = 2; j<argc; ++j ) {
    states.push_back( argv[j] );
  }
  
  AbstractCall* call = CallFactory::makeCall( endpoint, states, AbstractCall::STATUS );
  
  string error;
  uid_t userID = ::getuid( );
  
  string usercert = string("/tmp/x509up_u") + boost::lexical_cast<string>( userID );
  string userkey  = string("/tmp/x509up_u") + boost::lexical_cast<string>( userID );  
  if ( !::getenv("NOAUTH") ) {
    if(!call->init( usercert, userkey, error ))
      {
	cerr << error << endl;
	return 1;
      }
  } else {
    if(!call->init("", "", error ))
      {
	cerr << error << endl;
	return 1;
      }
  }
  
  emi_es::client::comm::AbstractCall::SOAP_CALL_ERROR_CODES code;
  if(!call->execute( error, code ) ) {
    cerr << error << endl;
//    cerr << AbstractCall::SOAP_CALL_ERROR_CODES_STRING[code] << endl;
    return 1;
  }
  
  map<string, ActivityStatus> good;
  map<string, InternalBaseFault> bad;
  
  call->getResponse( good, bad );
  
  map<string, ActivityStatus>::const_iterator git = good.begin();
  map<string, InternalBaseFault>::const_iterator bit = bad.begin();
  
  for( ; git != good.end(); ++git )
    {
      cout << "JobID      = " << git->first<<endl;
      cout << "Status     = " << git->second.getStatusString() << endl;
      vector<string> attrs; 
      git->second.getStatusAttributesString(attrs);
      cout << "Attributes = {" << join(attrs, ", ") << "}" << endl;
      cout << "Timestamp  = " << git->second.getTimestampString( ) << endl;
      cout << "Description= " << git->second.getDescription( ) << endl;
    }
  
  for( ; bit != bad.end(); ++bit )
      {
	cout << "JobID      = " << bit->first<<endl;//getActivityID()<<endl;
	cout << "Message    = " << bit->second.Message<<endl;
	if(bit->second.Timestamp)
	  cout << "Timestamp  = " << time_to_string(*bit->second.Timestamp)<< endl;
	if(bit->second.Description)
	  cout << "Description= " << *bit->second.Description << endl;
	if(bit->second.FailureCode)
	  cout << "FailCode   = " << boost::lexical_cast<string>(*bit->second.FailureCode) << endl;
      }
  delete call;
}
